@@ -168,4 +168,11 @@ class WorkflowFailed(RuntimeError):
     """A workflow has failed."""
     def __init__(self, *args, **kwargs):
         self.causes = kwargs.pop('causes', []) or []
+        self.hide_traceback = kwargs.pop('hide_traceback', False)
         super(WorkflowFailed, self).__init__(*args, **kwargs)
+
+
+class InvalidBlueprintImport(RuntimeError):
+    def __init__(self, *args, **kwargs):
+        self.causes = kwargs.pop('causes', []) or []
+        super(InvalidBlueprintImport, self).__init__(*args, **kwargs)
