@@ -43,14 +43,132 @@ def uninstall(ctx, ignore_failure=False, **kwargs):
         ignore_failure=ignore_failure)
 
 
+def _find_instances_to_heal(instances, healthy_instances):
+    """Examine instances, deciding which should be healed, or reinstalled.
+
+    Instances that are healthy, will not be touched at all - unless they're
+    part of a subgraph (ie. contained-in) of an instance that is going to be
+    reinstalled.
+    Instances that are not healthy will be healed if they define the heal
+    operation, or reinstalled otherwise.
+
+    :param instances: all instances to examine
+    :param healthy_instances: those that are already healthy - a subset
+        of instances
+    """
+
+    instances = set(instances)
+    to_heal = set()
+    to_reinstall = set()
+    while instances:
+        instance = instances.pop()
+        if instance in healthy_instances:
+            # instance is healthy! nothing to do. Its subgraph will still
+            # be examined
+            continue
+        elif instance.node.has_operation('cloudify.interfaces.lifecycle.heal'):
+            to_heal.add(instance)
+        else:
+            # instance doesn't have heal and is not healthy - it must be
+            # reinstalled, and its whole subgraph must be as well
+            subgraph = instance.get_contained_subgraph()
+            to_reinstall |= subgraph
+            to_heal -= subgraph
+            instances -= subgraph
+    return to_heal, to_reinstall
+
+
+def _find_healthy_instances(instances, require_task=False):
+    """Find instances that are already passing their status check
+
+    Depending on the require_task flag:
+      - if require_task is false, instances that are implicitly healthy
+        (eg. they have just been installed), are considered passing
+      - if require_task is true, only instances that are explicitly proven
+        to be healthy (their check_status operation is defined and has run),
+        are considered passing
+    """
+    healthy_instances = set()
+    for instance in instances:
+        try:
+            status = instance.system_properties['status']
+            if not status['ok']:
+                continue
+            if require_task and not status['task']:
+                continue
+            healthy_instances.add(instance)
+        except KeyError:
+            pass
+    return healthy_instances
+
+
+def _reinstall_disallowed_exception(to_reinstall):
+    if len(to_reinstall) > 5:
+        instance_ids = [inst.id for inst in to_reinstall]
+        names_message = (
+            ', '.join(instance_ids[:5]) +
+            ' (and {0} more)'.format(len(instance_ids) - 5)
+        )
+    else:
+        names_message = ', '.join(inst.id for inst in to_reinstall)
+    return RuntimeError(
+        'allow_reinstall is false, but would need to reinstall instances: '
+        + names_message
+    )
+
+
+def _find_heal_failed_instances(ctx, instances):
+    """Examine instances, and return the ones for which heal has failed.
+
+    To find the result of the heal operation, look in system-properties:
+    the heal subgraph's failure callback sets the flag there.
+    """
+    heal_failed = set()
+    for instance in instances:
+        try:
+            if instance.system_properties['heal_failed'] == ctx.execution_id:
+                heal_failed.add(instance)
+        except KeyError:
+            pass
+    return heal_failed
+
+
+def _clean_healed_property(ctx, instances):
+    """Remove the heal_failed flag from all instances.
+
+    No need to leave that flag around. When using it, we do check that it's
+    equal to the current execution id, so if we didn't clean it up, the
+    logic would still work - but it's nice to not leave unnecessary data
+    around.
+    """
+    for instance in instances:
+        system_properties = instance.system_properties or {}
+        if 'heal_failed' not in system_properties:
+            continue
+        del system_properties['heal_failed']
+        ctx.update_node_instance(
+            instance.id,
+            force=True,
+            system_properties=system_properties
+        )
+
+
 @workflow
 def auto_heal_reinstall_node_subgraph(
-        ctx,
-        node_instance_id,
-        diagnose_value='Not provided',
-        ignore_failure=True,
-        **kwargs):
-    """Reinstalls the whole subgraph of the system topology
+    ctx,
+    node_instance_id=None,
+    diagnose_value='Not provided',
+    ignore_failure=True,
+    check_status=True,
+    allow_reinstall=True,
+    force_reinstall=False,
+    **kwargs
+):
+    """Heals a subgraph of the system topology.
+
+    To heal a subgraph, run the heal operation on all the nodes,
+    and fall back to reinstalling the nodes if the heal operation is
+    not defined or fails.
 
     The subgraph consists of all the nodes that are hosted in the
     failing node's compute and the compute itself.
@@ -60,26 +178,102 @@ def auto_heal_reinstall_node_subgraph(
     :param node_instance_id: node_instances to reinstall
     :param diagnose_value: diagnosed reason of failure
     :param ignore_failure: ignore operations failures in uninstall workflow
+    :param check_status: run the status check before healing
+    :param allow_reinstall: if a heal operation fails or is not defined,
+        attempt reinstalling the instance
+    :param force_reinstall: don't even attempt to heal, always reinstall
     """
 
-    ctx.logger.info("Starting 'heal' workflow on {0}, Diagnosis: {1}".format(
-        node_instance_id, diagnose_value))
-    failing_node = ctx.get_node_instance(node_instance_id)
-    if failing_node is None:
-        raise ValueError('No node instance with id `{0}` was found'.format(
-            node_instance_id))
-    failing_node_host = ctx.get_node_instance(
-        failing_node._node_instance.host_id)
-    if failing_node_host is None:
-        subgraph_node_instances = failing_node.get_contained_subgraph()
+    if node_instance_id:
+        ctx.logger.info("Starting 'heal' workflow on %s, Diagnosis: %s",
+                        node_instance_id, diagnose_value)
+        failing_node = ctx.get_node_instance(node_instance_id)
+        if failing_node is None:
+            raise ValueError('No node instance with id `{0}` was found'
+                             .format(node_instance_id))
+
+        failing_node_host = ctx.get_node_instance(
+            failing_node._node_instance.host_id)
+        if failing_node_host is None:
+            sub_node_instances = failing_node.get_contained_subgraph()
+        else:
+            sub_node_instances = failing_node_host.get_contained_subgraph()
     else:
-        subgraph_node_instances = failing_node_host.get_contained_subgraph()
-    intact_nodes = set(ctx.node_instances) - subgraph_node_instances
+        ctx.logger.info("Starting 'heal' workflow for '%s' deployment, "
+                        "Diagnosis: %s", ctx.deployment.id, diagnose_value)
+        sub_node_instances = set()
+        for ni in ctx.node_instances:
+            ni_host = ctx.get_node_instance(ni._node_instance.host_id)
+            if ni_host is None:
+                sub_node_instances.update(ni.get_contained_subgraph())
+            else:
+                sub_node_instances.update(ni_host.get_contained_subgraph())
+
     graph = ctx.graph_mode()
-    lifecycle.reinstall_node_instances(graph=graph,
-                                       node_instances=subgraph_node_instances,
-                                       related_nodes=intact_nodes,
-                                       ignore_failure=ignore_failure)
+    if force_reinstall:
+        return lifecycle.reinstall_node_instances(
+            graph=graph,
+            node_instances=sub_node_instances,
+            related_nodes=set(ctx.node_instances) - sub_node_instances,
+            ignore_failure=ignore_failure,
+        )
+
+    if check_status:
+        status_graph = _make_check_status_graph(
+            ctx,
+            node_instance_ids=[ni.id for ni in sub_node_instances],
+            name='check_status',
+            ignore_failure=True,
+        )
+        status_graph.execute()
+
+        ctx.refresh_node_instances()
+
+    healthy_instances = _find_healthy_instances(
+        sub_node_instances,
+        # require_task if we're explicitly going to target a node instance.
+        # This is compatible with pre-6.4 heal.
+        # This means a targeted heal assumes nodes to be unhealthy unless
+        # proven healthy, and a deployment-wide heal assumes nodes healthy
+        # unless proven unhealthy.
+        require_task=bool(node_instance_id),
+    )
+    to_heal, to_reinstall = _find_instances_to_heal(
+        sub_node_instances,
+        healthy_instances,
+    )
+
+    if to_reinstall and not allow_reinstall:
+        # we already know we'll need to reinstall, so we can abort early
+        raise _reinstall_disallowed_exception(to_reinstall)
+
+    try:
+        lifecycle.heal_node_instances(
+            graph=graph,
+            node_instances=list(to_heal),
+            related_nodes=set(ctx.node_instances) - to_heal,
+        )
+    except Exception as e:
+        # error running the heal - we can still fall back to reinstalling!
+        ctx.logger.error('Error running heal: %s', e)
+
+    ctx.refresh_node_instances()
+    failed_heal = _find_heal_failed_instances(
+        ctx,
+        {ctx.get_node_instance(ni.id) for ni in to_heal},
+    )
+    _clean_healed_property(ctx, failed_heal)
+    to_reinstall |= failed_heal
+
+    if to_reinstall and not allow_reinstall:
+        raise _reinstall_disallowed_exception(to_reinstall)
+    if to_reinstall:
+        lifecycle.reinstall_node_instances(
+            graph=graph,
+            node_instances=to_reinstall,
+            related_nodes=set(ctx.node_instances) - to_reinstall,
+            ignore_failure=ignore_failure,
+        )
 
 
 def get_groups_with_members(ctx):
@@ -109,7 +303,7 @@ def get_groups_with_members(ctx):
 def _check_for_too_many_exclusions(exclude_instances, available_instances,
                                    delta, groups_members):
     """
-        Check whether the amount of exluded instances will make it possible to
+        Check whether the amount of excluded instances will make it possible to
         scale down by the given delta.
 
         :param exclude_instances: A list of node instance IDs to exclude.
@@ -279,7 +473,7 @@ def validate_inclusions_and_exclusions(include_instances, exclude_instances,
         raise RuntimeError(error_message)
 
 
-@workflow
+@workflow(resumable=True)
 def scale_entity(ctx,
                  scalable_entity_name,
                  delta,
@@ -389,34 +583,44 @@ def scale_entity(ctx,
 
     if abort_started:
         _abort_started_deployment_modifications(ctx, ignore_failure)
+    if ctx.resume:
+        mods = ctx.deployment.list_started_modifications()
+        if len(mods) > 1:
+            raise RuntimeError(
+                'scale can only be resumed when there is only one started '
+                'deployment modification, but found {0}'
+                .format(len(mods))
+            )
+        modification = mods[0]
+    else:
+        modification = ctx.deployment.start_modification({
+            scale_id: {
+                'instances': planned_num_instances,
+                'removed_ids_exclude_hint': exclude_instances,
+                'removed_ids_include_hint': include_instances,
 
-    modification = ctx.deployment.start_modification({
-        scale_id: {
-            'instances': planned_num_instances,
-            'removed_ids_exclude_hint': exclude_instances,
-            'removed_ids_include_hint': include_instances,
+                # While these parameters are now exposed, this comment is being
+                # kept as it provides useful insight into the hints
+                # These following parameters are not exposed at the moment,
+                # but should be used to control which node instances get
+                # scaled in(when scaling in).
+                # They are mentioned here, because currently, the
+                # modification API is not very well documented.
+                # Special care should be taken because if
+                # `scale_compute == True` (which is the default), then these
+                # ids should be the compute node instance ids which are
+                # not necessarily instances of the node specified by
+                # `scalable_entity_name`.
 
-            # While these parameters are now exposed, this comment is being
-            # kept as it provides useful insight into the hints
-            # These following parameters are not exposed at the moment,
-            # but should be used to control which node instances get scaled in
-            # (when scaling in).
-            # They are mentioned here, because currently, the modification API
-            # is not very documented.
-            # Special care should be taken because if `scale_compute == True`
-            # (which is the default), then these ids should be the compute node
-            # instance ids which are not necessarily instances of the node
-            # specified by `scalable_entity_name`.
+                # Node instances denoted by these instance ids should be
+                # *kept* if possible.
+                # 'removed_ids_exclude_hint': [],
 
-            # Node instances denoted by these instance ids should be *kept* if
-            # possible.
-            # 'removed_ids_exclude_hint': [],
-
-            # Node instances denoted by these instance ids should be *removed*
-            # if possible.
-            # 'removed_ids_include_hint': []
-        }
-    })
+                # Node instances denoted by these instance ids should
+                # be *removed* if possible.
+                # 'removed_ids_include_hint': []
+            }
+        })
     graph = ctx.graph_mode()
     try:
         ctx.logger.info('Deployment modification started. '
@@ -744,19 +948,29 @@ def check_status_on_failure(task):
     return HandlerResult.fail()
 
 
+def check_status_on_failure_ignore_failure(task):
+    """check_status failure handler that ignores the error"""
+    _set_node_instance_status(task)
+    return HandlerResult.ignore()
+
+
 @make_or_get_graph
 def _make_check_status_graph(
     ctx,
     run_by_dependency_order=False,
     type_names=None,
     node_ids=None,
-    node_instance_ids=None
+    node_instance_ids=None,
+    ignore_failure=False,
 ):
     """Make the graph for the check_status workflow
 
     This is very similar to execute_operation, but a bit simpler, because
     we only run a single operation from here.
     """
+    on_fail = check_status_on_failure
+    if ignore_failure:
+        on_fail = check_status_on_failure_ignore_failure
     graph = ctx.graph_mode()
     tasks = {}
     filtered_node_instances = _filter_node_instances(
@@ -779,7 +993,7 @@ def _make_check_status_graph(
         )
         task.info = {'instance_id': instance.id}
         task.on_success = check_status_on_success
-        task.on_failure = check_status_on_failure
+        task.on_failure = on_fail
         graph.add_task(task)
         tasks[instance.id] = task
 
@@ -818,13 +1032,17 @@ def _make_check_drift_graph(
     run_by_dependency_order=False,
     type_names=None,
     node_ids=None,
-    node_instance_ids=None
+    node_instance_ids=None,
+    ignore_failure=False,
 ):
     """Make the graph for the check_status workflow
 
     This is very similar to execute_operation, but a bit simpler, because
     we only run a single operation from here.
     """
+    on_fail = check_drift_on_failure
+    if ignore_failure:
+        on_fail = check_drift_on_failure_ignore_failure
     graph = ctx.graph_mode()
     tasks = {}
     filtered_node_instances = _filter_node_instances(
@@ -847,7 +1065,7 @@ def _make_check_drift_graph(
         )
         task.info = {'instance_id': instance.id}
         task.on_success = check_drift_on_success
-        task.on_failure = check_drift_on_failure
+        task.on_failure = on_fail
         graph.add_task(task)
         tasks[instance.id] = task
         for rel in instance.relationships:
@@ -856,7 +1074,7 @@ def _make_check_drift_graph(
             target_task = rel.execute_target_operation(
                 'cloudify.interfaces.relationship_lifecycle.check_drift')
             source_task.on_success = check_drift_on_success
-            source_task.on_failure = check_drift_on_failure
+            source_task.on_failure = on_fail
             graph.add_task(source_task)
             graph.add_dependency(source_task, task)
             source_task.info = {
@@ -865,7 +1083,7 @@ def _make_check_drift_graph(
             }
 
             target_task.on_success = check_drift_on_success
-            target_task.on_failure = check_drift_on_failure
+            target_task.on_failure = on_fail
             graph.add_task(target_task)
             graph.add_dependency(target_task, task)
             target_task.info = {
@@ -924,6 +1142,12 @@ def check_drift_on_failure(task):
     return HandlerResult.fail()
 
 
+def check_drift_on_failure_ignore_failure(task):
+    """check_drift failure handler that ignores the error"""
+    _set_node_instance_drift(task)
+    return HandlerResult.ignore()
+
+
 @workflow(resumable=True)
 def check_drift(ctx, *args, **kwargs):
     graph = _make_check_drift_graph(
