@@ -22,6 +22,7 @@ import mock
 import testtools
 
 from cloudify import exceptions
+from cloudify.plugins import lifecycle
 from cloudify.decorators import operation
 from cloudify.test_utils import workflow_test
 from cloudify.workflows.workflow_context import (Modification,
@@ -533,6 +534,266 @@ class TestSubgraphWorkflowLogic(testtools.TestCase):
         self.assertEqual(4, len(all_invocations))
 
 
+class TestHealOperation(testtools.TestCase):
+    """Tests for the heal workflow, using the heal operation.
+
+    Each test case runs the heal operation on a node instance, and checks
+    what operations were actually run. See the comments in the blueprint
+    itself for details about why are these operations expected in each case.
+    """
+
+    def _do_test_heal(self, env, node_id, **parameters):
+        instances = {ni.node_id: ni for ni in env.storage.get_node_instances()}
+
+        parameters.setdefault('node_instance_id', instances[node_id].id)
+        env.execute('heal', parameters=parameters)
+        return {
+            ni.node_id: {
+                inv['operation']
+                for inv in ni.runtime_properties.get('invocations', [])
+            } for ni in env.storage.get_node_instances()
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_heal_operation(self, env):
+        invocations = self._do_test_heal(env, 'node1')
+        assert invocations['node1'] == {'cloudify.interfaces.lifecycle.heal'}
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_failing_status_heal_operation(self, env):
+        invocations = self._do_test_heal(env, 'node2')
+        assert invocations['node2'] == {
+            'cloudify.interfaces.lifecycle.heal',
+            'cloudify.interfaces.validation.check_status',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_passing_status_heal_operation(self, env):
+        invocations = self._do_test_heal(env, 'node3')
+        assert invocations['node3'] == {
+            'cloudify.interfaces.validation.check_status',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_failing_heal(self, env):
+        invocations = self._do_test_heal(env, 'node4')
+        assert invocations['node4'] == {
+            'cloudify.interfaces.lifecycle.create',
+            'cloudify.interfaces.lifecycle.heal',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_no_heal(self, env):
+        invocations = self._do_test_heal(env, 'node5')
+        assert invocations['node5'] == {
+            'cloudify.interfaces.lifecycle.create',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_heal_contained(self, env):
+        invocations = self._do_test_heal(env, 'node6')
+        assert invocations['node6'] == {
+            'cloudify.interfaces.lifecycle.heal',
+        }
+        assert invocations['node6_contained'] == {
+            'cloudify.interfaces.lifecycle.heal',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_reinstall_contained_with_heal(self, env):
+        invocations = self._do_test_heal(env, 'node7')
+        assert invocations['node7'] == {
+            'cloudify.interfaces.lifecycle.create',
+        }
+        assert invocations['node7_contained'] == {
+            'cloudify.interfaces.lifecycle.create',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_only_contained_fails_status(self, env):
+        invocations = self._do_test_heal(env, 'node8')
+        assert invocations['node8'] == {
+            'cloudify.interfaces.validation.check_status',
+        }
+        assert invocations['node8_contained'] == {
+            'cloudify.interfaces.validation.check_status',
+            'cloudify.interfaces.lifecycle.heal',
+        }
+        assert invocations['node8_contained2'] == {
+            'cloudify.interfaces.validation.check_status',
+            'cloudify.interfaces.lifecycle.create',
+        }
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_force_reinstall(self, env):
+        invocations = self._do_test_heal(env, 'node1', force_reinstall=True)
+        assert invocations['node1'] == {'cloudify.interfaces.lifecycle.create'}
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_dont_allow_reinstall(self, env):
+        instances = {ni.node_id: ni for ni in env.storage.get_node_instances()}
+        with self.assertRaisesRegex(RuntimeError, 'allow_reinstall'):
+            env.execute('heal', parameters={
+                'node_instance_id': instances['node4'].id,
+                'allow_reinstall': False,
+            })
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_without_check_status(self, env):
+        ni = env.storage.get_node_instances(node_id='node2')[0]
+        system_properties = ni.system_properties
+        system_properties['status'] = {
+            'ok': True,
+            'task': 'fake task!',
+        }
+        env.storage.update_node_instance(
+            ni.id,
+            system_properties=system_properties,
+            version=ni.version,
+        )
+        # the status check would fail! but let's say the status is already
+        # set to OK, and then, we'll do nothing
+        invocations = self._do_test_heal(env, 'node2', check_status=False)
+        assert invocations['node2'] == set()
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_reinstall_long_message(self, env):
+        instances = {ni.node_id: ni for ni in env.storage.get_node_instances()}
+        with self.assertRaisesRegex(RuntimeError, r'and \d+ more'):
+            env.execute('heal', parameters={
+                'node_instance_id': instances['node9'].id,
+                'allow_reinstall': False,
+            })
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-heal-operation.yaml'))
+    def test_node_instance_id_not_provided(self, env):
+        env.execute('heal', parameters={'check_status': True})
+        invocations = [
+            invocations for ni in env.storage.get_node_instances()
+            for invocations in ni.runtime_properties.get('invocations', [])
+        ]
+        assert set(invocation['node_id']
+                   for invocation in invocations
+                   if invocation.get('operation')
+                   == 'cloudify.interfaces.validation.check_status') == \
+               {'node2', 'node3', 'node8',
+                'node8_contained', 'node8_contained2', }
+        assert set(invocation['node_id']
+                   for invocation in invocations
+                   if invocation.get('operation')
+                   == 'cloudify.interfaces.lifecycle.heal') == \
+               {'node2', 'node8_contained'}
+
+
+def update_test_workflow(ctx, node_instance_id, **kwargs):
+    instance = ctx.get_node_instance(node_instance_id)
+    lifecycle.update_node_instances(
+        graph=ctx.graph_mode(),
+        node_instances={instance},
+        related_nodes=set(ctx.node_instances) - {instance},
+    )
+
+
+class TestUpdateOperation(testtools.TestCase):
+    """Test the update flow of the lifecycle processor.
+
+    This is using a "fake" workflow to run the lifecycle code; the real
+    workflow is defined in cloudify-system-workflows instead.
+    """
+    def _do_test_update(self, env, node_id, **parameters):
+        instances = {ni.node_id: ni for ni in env.storage.get_node_instances()}
+
+        parameters.setdefault('node_instance_id', instances[node_id].id)
+        env.execute('test_update', parameters=parameters)
+        return {
+            ni.node_id: [
+                inv['operation']
+                for inv in ni.runtime_properties.get('invocations', [])
+            ] for ni in env.storage.get_node_instances()
+        }
+
+    def _get_ni(self, env, node_id):
+        """Get the only node instance of the given node"""
+        instances = env.storage.get_node_instances(node_id=node_id)
+        assert len(instances) == 1
+        return instances[0]
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-update-operation.yaml'))
+    def test_update_operation(self, env):
+        ni = self._get_ni(env, 'node1')
+        env.storage.update_node_instance(ni.id, system_properties={
+            'configuration_drift': {'result': True},
+        }, force=True, version=0)
+
+        invocations = self._do_test_update(env, 'node1')
+        assert invocations['node1'] == ['cloudify.interfaces.lifecycle.update']
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-update-operation.yaml'))
+    def test_update_fails(self, env):
+        ni = self._get_ni(env, 'node2')
+        env.storage.update_node_instance(ni.id, system_properties={
+            'configuration_drift': {'result': True},
+        }, force=True, version=0)
+
+        # the operation fails, but it doesn't actually fail the whole workflow;
+        # instead, the failure is recorded in system-properties
+        invocations = self._do_test_update(env, 'node2')
+        assert invocations['node2'] == ['cloudify.interfaces.lifecycle.update']
+        ni = env.storage.get_node_instances(node_id='node2')[0]
+        assert ni.system_properties.get('update_failed')
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-update-operation.yaml'))
+    def test_update_clears_drift(self, env):
+        ni = self._get_ni(env, 'node1')
+        env.storage.update_node_instance(ni.id, system_properties={
+            'configuration_drift': {'result': True},
+        }, force=True, version=0)
+        invocations = self._do_test_update(env, 'node1')
+        assert invocations['node1'] == ['cloudify.interfaces.lifecycle.update']
+        ni = env.storage.get_node_instances(node_id='node1')[0]
+        assert not ni.system_properties.get('configuration_drift')
+
+    @workflow_test(path.join(
+        'resources', 'blueprints', 'test-update-operation.yaml'))
+    def test_update_relationship_operation(self, env):
+        node1_ni = self._get_ni(env, 'node1')
+        ni = self._get_ni(env, 'node3')
+        env.storage.update_node_instance(ni.id, system_properties={
+            'source_relationships_configuration_drift': {
+                node1_ni.id: {'result': True},
+            },
+        }, force=True, version=0)
+        env.storage.update_node_instance(node1_ni.id, system_properties={
+            'target_relationships_configuration_drift': {
+                ni.id: {'result': True},
+            },
+        }, force=True, version=0)
+
+        invocations = self._do_test_update(env, 'node3')
+        assert invocations['node3'] == [
+            # source operation
+            'cloudify.interfaces.relationship_lifecycle.update',
+            # target operation
+            'cloudify.interfaces.relationship_lifecycle.update',
+        ]
+
+
 class TestRelationshipOrderInLifecycleWorkflows(testtools.TestCase):
 
     blueprint_path = path.join('resources', 'blueprints',
@@ -670,7 +931,7 @@ class TestCheckStatus(unittest.TestCase):
         assert fail_instance['system_properties']['status']
         assert not fail_instance['system_properties']['status']['ok']
 
-        # with run_by_dependency_order, the noop instance didnt even run,
+        # with run_by_dependency_order, the noop instance didn't even run,
         # because it depends on a failing task
         noop_instance = noop_instances[0]
         assert 'status' not in noop_instance['system_properties']
@@ -843,6 +1104,37 @@ class TestRollbackWorkflow(LifecycleBaseTest):
         assert node_a_time < node_b_time
 
 
+class TestInputTypes(LifecycleBaseTest):
+    @workflow_test(path.join(
+        'resources',
+        'blueprints',
+        'test-operation-input-types.yaml'))
+    def test_valid_input_types(self, cfy_local):
+        cfy_local.execute(
+            'execute_operation',
+            {'operation': 'cloudify.interfaces.lifecycle.create',
+             'operation_kwargs': {'v_integer': 33},
+             'allow_kwargs_override': 'True'
+             }
+        )
+
+    @workflow_test(path.join(
+        'resources',
+        'blueprints',
+        'test-operation-input-types.yaml'))
+    def test_invalid_input_types(self, cfy_local):
+        self.assertRaisesRegex(
+            RuntimeError,
+            r'does not match.*integer$',
+            cfy_local.execute,
+            'execute_operation',
+            {'operation': 'cloudify.interfaces.lifecycle.create',
+             'operation_kwargs': {'v_integer': 'A lazy dog'},
+             'allow_kwargs_override': 'True'
+             }
+        )
+
+
 @operation
 def exec_op_test_operation(ctx, **kwargs):
     ctx.instance.runtime_properties['test_op_visited'] = True
