@@ -19,7 +19,6 @@ import tempfile
 import copy
 import importlib
 import shutil
-import uuid
 import json
 import time
 import threading
@@ -97,7 +96,7 @@ class _Environment(object):
                                      ', '.join(workflows)))
 
         workflow = workflows[workflow_name]
-        execution_id = str(uuid.uuid4())
+        execution_id = utils.uuid4()
         ctx = {
             'type': 'workflow',
             'local': True,
@@ -300,7 +299,8 @@ def _merge_and_validate_execution_parameters(
 
         if 'default' not in param:
             if name not in execution_parameters:
-                missing_mandatory_parameters.add(name)
+                if param.get('required', True):
+                    missing_mandatory_parameters.add(name)
                 continue
             merged_parameters[name] = execution_parameters[name]
         else:
@@ -347,11 +347,10 @@ class DeploymentStorage(object):
     @contextmanager
     def _lock(self, instance_id):
         with self._main_instances_lock:
-            if instance_id not in self._instance_locks:
+            instance_lock = self._instance_locks.get(instance_id)
+            if instance_lock is None:
                 instance_lock = threading.RLock()
                 self._instance_locks[instance_id] = instance_lock
-            else:
-                instance_lock = self._instance_locks[instance_id]
         with instance_lock:
             yield
 
@@ -561,13 +560,14 @@ class _Storage(ABC):
             'plan': deployment_plan,
             'nodes': {n.id: n for n in nodes},
             'created_at': datetime.utcnow(),
-            'inputs': inputs,
+            'inputs': inputs or {},
             'workflows': workflows,
             'scaling_groups': deployment_plan['scaling_groups'],
         }
         self.store_deployment(name, deployment)
         for instance in node_instances:
             self.store_instance(name, NodeInstance(instance))
+        return self.load(name)
 
     def load(self, name):
         dep = self.get_deployment(name)
@@ -655,8 +655,23 @@ class _Storage(ABC):
     def store_executions(self, deployment_id, executions):
         raise NotImplementedError()
 
-    @abc.abstractmethod
     def create_deployment_update(self, deployment_id, update_id, update):
+        dep_update = {
+            'id': update_id,
+            'deployment_id': deployment_id,
+            'old_blueprint_id': None,
+            'new_blueprint_id': None,
+            'old_inputs': {},
+            'new_inputs': {},
+            'steps': [],
+            'runtime_only_evaluation': True,
+        }
+        dep_update.update(update)
+        self.store_deployment_update(deployment_id, update_id, dep_update)
+        return self.get_deployment_update(deployment_id, update_id)
+
+    @abc.abstractmethod
+    def store_deployment_update(self, deployment_id, update_id, update):
         raise NotImplementedError()
 
     @abc.abstractmethod
@@ -754,7 +769,7 @@ class InMemoryStorage(_Storage):
     def store_executions(self, deployment_id, executions):
         self._executions[deployment_id] = executions
 
-    def create_deployment_update(self, deployment_id, update_id, update):
+    def store_deployment_update(self, deployment_id, update_id, update):
         self._deployment_updates[(deployment_id, update_id)] = update
 
     def get_deployment_update(self, deployment_id, update_id):
@@ -947,7 +962,7 @@ class FileStorage(_Storage):
         with open(executions_path, 'w') as f:
             json.dump(executions, f, indent=4, cls=JSONEncoderWithDatetime)
 
-    def create_deployment_update(self, deployment_id, update_id, update):
+    def store_deployment_update(self, deployment_id, update_id, update):
         updates_dir = os.path.join(
             self._root_storage_dir,
             'deployments',
