@@ -42,6 +42,7 @@ NAMESPACES_MAPPING = 'namespaces_mapping'
 CONSTRAINTS = 'constraints'
 DEFAULT = 'default'
 TYPE = 'type'
+ITEM_TYPE = 'item_type'
 LABELS = 'labels'
 BLUEPRINT_LABELS = 'blueprint_labels'
 DEFAULT_SCHEDULES = 'default_schedules'
@@ -95,8 +96,18 @@ VALIDATE_DEFINITIONS_VERSION = 'validate_definitions_version'
 RESOLVER_IMPLEMENTATION_KEY = 'implementation'
 RESLOVER_PARAMETERS_KEY = 'parameters'
 
+TYPES_BASED_ON_DB_ENTITIES = [
+    'blueprint_id', 'deployment_id', 'capability_value', 'scaling_group',
+    'node_id', 'node_type', 'node_instance', 'secret_key'
+]
+TYPES_WHICH_REQUIRE_DEPLOYMENT_ID_CONSTRAINT = [
+    'capability_value', 'scaling_group',
+    'node_id', 'node_type', 'node_instance'
+]
 USER_PRIMITIVE_TYPES = ['string', 'integer', 'float', 'boolean', 'list',
-                        'dict', 'regex', 'secret']
+                        'dict', 'regex', 'textarea'] \
+                       + TYPES_BASED_ON_DB_ENTITIES
+
 PLUGIN_DSL_KEYS_NOT_FROM_YAML = ['blueprint_labels', 'labels', 'resource_tags']
 PLUGIN_DSL_KEYS_READ_FROM_DB = PLUGIN_DSL_KEYS_NOT_FROM_YAML
 PLUGIN_DSL_KEYS_ADD_VALUES_NODE = ['blueprint_labels', 'labels']
